/*BEGIN COPYRIGHT */
/**********************************************************************
*
*   Licensed Materials - Property of IBM
*   5697-FM3 5655-A96
*   (C) Copyright IBM Corp. 1998, 2000
*   US Government Users Restricted Rights - Use, duplication or
*   disclosure restricted by GSA ADP Schedule Contract
*   with IBM Corp.
*
**********************************************************************/
/*END COPYRIGHT */

/* BEGIN DISCLAIMER */
/**********************************************************************
*  DISCLAIMER
*  ----------
*  This material contains programming source code for your consideration.
*  These examples have not been thoroughly tested under all conditions.
*  IBM, therefore, cannot guarantee or imply reliability, serviceability,
*  or function of these programs.
*  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
*  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
*  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
*  FOR A PARTICULAR PURPOSE.
**********************************************************************/
/* END DISCLAIMER */

/*********************************************************************/
/* MQSeries Workflow API Tutorial - Persistent lists and service     */
/*                                  queries                          */
/*                                                                   */
/* This program demonstrates                                         */
/* 1. the handling of persistent lists. Defining a persistent list   */
/*    means defining a persistent view to a collection of objects.   */
/* 2. the handling of process template lists, process instance lists */
/*    and worklists queried from the execution service               */
/*********************************************************************/

#include <fmctjinc.hxx>                 // compiler-dependent #includes
#include <fmcjprun.hxx>                 // MQWF C++ API interface
#include <ctype.h>                      // character classification


int main(int argc, char ** argv)

{

  bool                                  del             = false;

  unsigned long                         i               = 0;
  unsigned long                         j               = 0;
  unsigned long                         threshold       = 10;

  string                                user            = "";
  string                                listName        = "";

  FmcjExecutionService                  service;


/*--------------------------------------------------------------------*/
/* Check command line parameters, logon to system                     */
/*--------------------------------------------------------------------*/

  if ( argc<3 || argc>4 )
  {
    cout << "\nUsage: " << argv[0] << " [-d] <UserID> <Password>" << endl;
    cout <<   "       -d: Delete persistent lists created by this tool" << endl;
    return -1;
  }
  cout << endl;
  APIRET rc = FmcjGlobal::Connect();
  FmcjResult * result = FmcjResult::ObjectOfCurrentThread();

  if (argc==4) if (strncmp(argv[1],"-d",2)==0) del = true;
               else cout << "Ignoring unknown option: " << argv[1] << endl;
  if (argc==3) rc = service.Logon( argv[1], argv[2], FmcjService::Default );
  else         rc = service.Logon( argv[2], argv[3], FmcjService::Default );

  if (rc != FMC_OK)
  {
    cout << "Logon failed with RC " << rc << " - " << result->MessageText()
         << endl;
    // Deinit API environment
    FmcjGlobal::Disconnect();
    delete(result);
    return -1;
  }

  user = service.UserID();

  cout << "Logged on with UserID '" << user
       << "' on system '" << service.SystemName()
       << "' at system group '" << service.SystemGroupName()
       << "'." << endl;


/*********************************************************************/
/* Demonstrate handling of process instance lists.                   */
/*                                                                   */
/* Create process instance lists with different properties suited    */
/* for different scenarios. If a list with the name 'Sample*' already*/
/* exists, it will be deleted before list creation starts.           */
/*********************************************************************/

  FmcjProcessInstanceList               instanceList;
  vector<FmcjProcessInstanceList>       iList;
  FmcjProcessInstance                   processInstance;
  vector<FmcjProcessInstance>           viList;

/*-------------------------------------------------------------------*/
/* Clean up existing process instance lists.                         */
/*-------------------------------------------------------------------*/

  rc= service.QueryProcessInstanceLists( iList );
  if (rc != FMC_OK)
  {
    cout << "Querying process instance lists failed with RC "
         << rc << " - " << result->MessageText() << endl;
    service.Logoff();
    // Deinit API environment
    FmcjGlobal::Disconnect();
    delete(result);
    return -1;
  }

  for( i=0; i< iList.size(); i++ )
  {
    instanceList = iList[i];
    listName = iList[i].Name();

    if (strncmp(listName.c_str(),"Sample",6)==0)
    {
      rc= instanceList.Delete();
      if (rc != FMC_OK)
      {
        cout << "\nDeletion of process instance list '" << instanceList.Name()
             << "' failed with RC " << rc << " - " << result->MessageText() << endl;
      }
      else if (del) cout << "\nProcess instance list deleted: "
                         << listName << endl;
    }
  }

/*-------------------------------------------------------------------*/
/* Create process instance lists for different purposes.             */
/* 1. Create a list which will contain only running process          */
/*    instances when queried and which is accessible by everyone.    */
/*    Show contents of created list.                                 */
/* 2. Create a list containing the latest started process instance.  */
/*    This list is for the personal use of the logged on user only.  */
/*    Show contents of created list.                                 */
/*-------------------------------------------------------------------*/

  if (!del) {                        // Only create lists when -d switch
                                     // was not used
  threshold = 5;
  string listName = "Sample_RunningProcessInstances";
  string desc = "Sample list of all process instances which are currently running";
  string PIListFilter = "STATE = RUNNING";
  string PIListSort     = "NAME ASC";
  rc= service.CreateProcessInstanceList( listName,
                                         FmcjPersistentList::Public,
                                         0,
                                         &desc,
                                         &PIListFilter,
                                         &PIListSort,
                                         &threshold,
                                         instanceList );
  if (rc != FMC_OK)
  {
    cout << "\nCreation of public process instance list '" << listName
         << "' failed with RC " << rc << " - " << result->MessageText() << endl;
  }
  else cout << "\nPublic process instance list created: " << listName << endl;

  // clean up vector and query process instance lists again

  iList.erase(iList.begin(),iList.end());

  rc= service.QueryProcessInstanceLists( iList );
  if (rc != FMC_OK)
  {
    cout << "Querying process instance lists failed with RC " << rc << " - "
         << result->MessageText() << endl;
    service.Logoff();
    // Deinit API environment
    FmcjGlobal::Disconnect();
    delete(result);
    return -1;
  }

  // Select the newly created process instance list

  if (iList.size() > 0 )
  {
    for( i=0; i<iList.size(); i++ )
    {
      instanceList = iList[i];
      listName = instanceList.Name();

      if (strncmp(listName.c_str(),"Sample_RunningProcessInstances",30)==0)
      {
       cout << "\nShowing process instances matched by this list:\n" << endl;

       rc = instanceList.QueryProcessInstances( viList );
       if (rc != FMC_OK)
       {
         cout << "Querying process instances failed with RC " << rc << " - "
              << result->MessageText() << endl;
         service.Logoff();
         // Deinit API environment
         FmcjGlobal::Disconnect();
         delete(result);
         return -1;
       }

       if ( viList.size() > 0 )
       {
         for ( j=0; j<viList.size(); j++ )
         {
           cout << "     " << viList[j].Name() << endl;
         }
       }
       else cout << "     <none>" << endl;

      }
    }
   }

/*-------------------------------------------------------------------*/
/* Create a list containing only the latest started process instance.*/
/* We cannot do this very smooth with a filter, so we use sort and   */
/* threshold options.                                                */

   listName = "Sample_LatestStartedInstance";
   desc = "Sample list which contains only the latest started process instance";
   PIListSort = "START_TIME DESC";
   threshold= 1; /* The server first sorts and then applies the threshold */

   rc= service.CreateProcessInstanceList( listName,
                                          FmcjPersistentList::Private,
                                          &user,
                                          &desc,
                                          FmcjNoFilter,
                                          &PIListSort,
                                          &threshold,
                                          instanceList );
   if (rc != FMC_OK)
   {
     cout << "\nCreation of private process instance list '" << listName
          << "'\nfor '" << user << "' failed with RC " << rc << " - "
          << result->MessageText() << endl;
   }
   else cout << "\nPrivate process instance list for '" << user
             << "' created: " << listName << endl;
  }

  // Clean up vectors and query process instance lists again

  iList.erase(iList.begin(),iList.end());
  viList.erase(viList.begin(),viList.end());

  rc= service.QueryProcessInstanceLists( iList );
  if (rc != FMC_OK)
  {
    cout << "Querying process instance lists failed with RC " << rc << " - "
         << result->MessageText() << endl;
    service.Logoff();
    // Deinit API environment
    FmcjGlobal::Disconnect();
    delete(result);
    return -1;
  }

  // Select the newly created process instance list

  for( i=0; i< iList.size(); i++ )
  {
    instanceList = iList[i];
    listName = instanceList.Name();

    if (strncmp(listName.c_str(),"Sample_LatestStartedInstance",28)==0)
    {
      cout << "\nShowing process instance matched by this list:\n" << endl;

      rc = instanceList.QueryProcessInstances( viList );
      if (rc != FMC_OK)
      {
        cout << "Querying process instances failed with RC " << rc << " - "
             << result->MessageText() << endl;
        service.Logoff();
        // Deinit API environment
        FmcjGlobal::Disconnect();
        delete(result);
        return -1;
      }

      if ( viList.size() > 0 )
      {
        for ( j=0; j<viList.size(); j++ )
        {
          cout << "     " << viList[j].Name() << endl;
        }
      }
      else cout << "     <none>" << endl;
    }
  }


/*********************************************************************/
/* Demonstrate handling of worklists.                                */
/*                                                                   */
/* Create worklists with different properties suited                 */
/* for different scenarios. If a list with the name 'Sample*' already*/
/* exists, it will be deleted before list creation starts.           */
/*********************************************************************/

  FmcjWorklist                          itemList;
  vector<FmcjWorklist>                  wList;
  FmcjWorkitem                          workitem;
  vector<FmcjWorkitem>                  vwList;

/*-------------------------------------------------------------------*/
/* Clean up existing worklists                                       */
/*-------------------------------------------------------------------*/

  rc= service.QueryWorklists( wList );
  if (rc != FMC_OK)
  {
    cout << "Querying worklists failed with RC " << rc << " - "
         << result->MessageText() << endl;
    service.Logoff();
    // Deinit API environment
    FmcjGlobal::Disconnect();
    delete(result);
    return -1;
  }

  for( i=0; i< wList.size(); i++ )
  {
    itemList = wList[i];
    listName = itemList.Name();

    if (strncmp(listName.c_str(),"Sample",6)==0)
    {
      rc= itemList.Delete();
      if (rc != FMC_OK)
      {
        cout << "\nDeletion of worklist '" << listName
             << "' failed with RC "
             << rc << " - "
             << result->MessageText()
             << endl;
      }
      else if (del) cout << "\nWorklist deleted: "
                         << listName << endl;
    }
  }

/*-------------------------------------------------------------------*/
/* Create worklists for different purposes.                          */
/* 1. Create a list containing notification items only.              */
/*    This list is intended for public use and has no owner.         */
/* 2. Create a list containing the user's oldest 20 workitems.       */
/*    This list is for the personal use of the logged on user only.  */
/*-------------------------------------------------------------------*/

  if (!del) {                        // only create lists when -d switch
  				     // was not used
  listName = "Sample_NotificationList";
  string desc = "Sample list of all notifications for the logged on user.";
  string WIListFilter = "TYPE IN (PROCESS_NOTIFICATION,FIRST_NOTIFICATION,SECOND_NOTIFICATION)";
  string WIListSort = "PRIORITY ASC";
  rc = service.CreateWorklist( listName,
                               FmcjPersistentList::Public,
                               0,
                               &desc,
                               &WIListFilter,
                               &WIListSort,
                               FmcjNoThreshold,
                               itemList );
  if (rc != FMC_OK)
  {
    cout << "\nCreation of public worklist '" << listName
         << "' failed with RC " << rc << " - " << result->MessageText() << endl;
  }
  else cout << "\nPublic worklist created: "
            << listName << endl;

  // Clean up vector and worklists again

  wList.erase(wList.begin(),wList.end());

  rc= service.QueryWorklists( wList );
  if (rc != FMC_OK)
  {
    cout << "Querying worklists failed with RC " << rc << " - "
         << result->MessageText() << endl;
    service.Logoff();
    // Deinit API environment
    FmcjGlobal::Disconnect();
    delete(result);
    return -1;
  }

  // Select the newly created worklist

  for( i=0; i<wList.size(); i++ )
  {
    itemList = wList[i];
    listName = itemList.Name();

    if (strncmp(listName.c_str(),"Sample_NotificationList",23)==0)
    {
      cout << "\nShowing workitem matched by this list:\n" << endl;

      rc = itemList.QueryWorkitems( vwList );
      if (rc != FMC_OK)
      {
        cout << "Querying workitems failed with RC " << rc << " - "
             << result->MessageText() << endl;
        service.Logoff();
        // Deinit API environment
        FmcjGlobal::Disconnect();
        delete(result);
        return -1;
      }

      if ( vwList.size() > 0 )
      {
        for ( j=0; j<vwList.size(); j++ )
        {
          cout << "     " << vwList[j].Name() << endl;
        }
      }
      else cout << "     <none>" << endl;

    }
  }

/*-------------------------------------------------------------------*/

  listName = "Sample_MyWork";
  desc = "Sample list of the logged on user's workitems";
  WIListSort = "LAST_MODIFICATION_TIME DESC";
  threshold= 20;

  rc= service.CreateWorklist( listName,
                              FmcjPersistentList::Private,
                              &user,
                              &desc,
                              FmcjNoFilter,
                              &WIListSort,
                              &threshold,
                              itemList );
  if (rc != FMC_OK)
  {
    cout << "\nCreation of private worklist '" << listName
         << "'\n for '" << user << "' failed with RC " << rc << " - "
         << result->MessageText() << endl;
  }
  else cout << "\nPrivate worklist for '" << user
    	     << "' created: " << listName << endl;

  } // end if: del==false

  // Clean up vector and worklists again

  wList.erase(wList.begin(),wList.end());
  vwList.erase(vwList.begin(),vwList.end());

  rc= service.QueryWorklists( wList );
  if (rc != FMC_OK)
  {
    cout << "Querying worklists failed with RC " << rc << " - "
         << result->MessageText() << endl;
    service.Logoff();
    // Deinit API environment
    FmcjGlobal::Disconnect();
    delete(result);
    return -1;
  }

  // select the newly created worklist

  for( i=0; i< wList.size(); i++ )
  {
    itemList = wList[i];
    listName = itemList.Name();

    if (strncmp(listName.c_str(),"Sample_MyWork",13)==0)
    {
      cout << "\nShowing workitems matched by this list:\n" << endl;

      rc = itemList.QueryWorkitems( vwList );
      if (rc != FMC_OK)
      {
        cout << "Querying workitems failed with RC " << rc << " - "
             << result->MessageText() << endl;
         service.Logoff();
         // Deinit API environment
         FmcjGlobal::Disconnect();
         delete(result);
         return -1;
      }

      if ( vwList.size() > 0 )
      {
        for ( j=0; j<vwList.size(); j++ )
        {
          cout << "     " << vwList[j].Name() << endl;
        }
      }
      else cout << "     <none>" << endl;

    }
  }


/*********************************************************************/
/* Demonstrate querying process templates                            */
/*                                                                   */
/* Query process templates with different properties suited          */
/* for different scenarios.					     */
/*********************************************************************/

  FmcjProcessTemplate		        processTemplate;
  vector<FmcjProcessTemplate>           tList;

/*-------------------------------------------------------------------*/
/* 1. Display all existing process templates			     */
/* 2. Display a selection of process templates using filter and      */
/*    threshold criteria.                                            */
/*-------------------------------------------------------------------*/

  cout << "\nDisplaying all existing process templates sorted by 'NAME ASC':\n"
       << endl;

  string PTListSort   = "NAME ASC";

  rc = service.QueryProcessTemplates( FmcjNoFilter,
  				      &PTListSort,
  				      FmcjNoThreshold,
  				      tList );

  if (rc != FMC_OK)
  {
    cout << "Querying process templates failed with RC " << rc << " - "
         << result->MessageText() << endl;
    service.Logoff();
    // Deinit API environment
    FmcjGlobal::Disconnect();
    delete(result);
    return -1;
  }

  if (tList.size() > 0 )
  {
    for( i=0; i<tList.size(); i++ )
    {
      processTemplate = tList[i];
      cout << "     " << processTemplate.Name() << endl;
    }
  }
  else cout << "     <none>" << endl;

  cout << "\nDisplaying process templates with filter "
       << "'NAME='CreditRequest',\n" << "threshold " << threshold
       << " and sort criterium 'NAME DESC'.\n"
       << endl;

  string PTListFilter = "NAME='CreditRequest'";
  PTListSort   = "NAME DESC";

  tList.erase(tList.begin(),tList.end());

  rc = service.QueryProcessTemplates( &PTListFilter,
  				      &PTListSort,
  				      &threshold,
  				      tList );

  if (rc != FMC_OK)
  {
    cout << "Querying process templates failed with RC " << rc << " - "
         << result->MessageText() << endl;
    service.Logoff();
    // Deinit API environment
    FmcjGlobal::Disconnect();
    delete(result);
    return -1;
  }

  if ( tList.size()>0 )
  {
    for( i=0; i<tList.size(); i++ )
    {
      processTemplate = tList[i];
      cout << "     " << processTemplate.Name() << endl;
    }
  }
  else cout << "     <none>" << endl;


/*********************************************************************/
/* Demonstrate querying process instances                            */
/*                                                                   */
/* Query process instances with different properties suited          */
/* for different scenarios.					     */
/*********************************************************************/

/*-------------------------------------------------------------------*/
/* 1. Display all existing process instances			     */
/* 2. Display a selection of process instances using filter and      */
/*    threshold criteria.                                            */
/*-------------------------------------------------------------------*/

  // Clear vector
  viList.erase(viList.begin(),viList.end());

  cout << "\nDisplaying all process instances sorted by 'NAME ASC':\n"
       << endl;

  string PIListSort   = "NAME ASC";

  rc = service.QueryProcessInstances( FmcjNoFilter,
  				      &PIListSort,
  				      FmcjNoThreshold,
  				      viList );

  if (rc != FMC_OK)
  {
    cout << "Querying process instances failed with RC " << rc << " - "
         << result->MessageText() << endl;
    service.Logoff();
    // Deinit API environment
    FmcjGlobal::Disconnect();
    delete(result);
    return -1;
  }

  if (viList.size() > 0 )
  {
    for( i=0; i<viList.size(); i++ )
    {
      processInstance = viList[i];
      cout << "     " << processInstance.Name() << endl;
    }
  }
  else cout << "     <none>" << endl;

  // Clear vector
  viList.erase(viList.begin(),viList.end());

  cout << "\nDisplaying process instances with filter "
       << "'STATE=RUNNING',\n" << "threshold " << threshold
       << " and sort criterium 'START_TIME DESC'.\n"
       << endl;

  string PIListFilter = "STATE IN (RUNNING)";
  PIListSort   = "START_TIME DESC";

  rc = service.QueryProcessInstances( &PIListFilter,
  				      &PIListSort,
  				      &threshold,
  				      viList );

  if (rc != FMC_OK)
  {
    cout << "Querying process instances failed with RC " << rc << " - "
         << result->MessageText() << endl;
    service.Logoff();
    // Deinit API environment
    FmcjGlobal::Disconnect();
    delete(result);
    return -1;
  }

  if ( viList.size()>0 )
  {
    for( i=0; i<viList.size(); i++ )
    {
      processInstance = viList[i];
      processInstance.Refresh();

      cout << string(processInstance.CreationTime()) << "   "
           << processInstance.Name() << endl;
    }
  }
  else cout << "     <none>" << endl;


/*********************************************************************/
/* Demonstrate querying workitems                                    */
/*                                                                   */
/* Query workitems with different properties suited                  */
/* for different scenarios.					     */
/*********************************************************************/

/*-------------------------------------------------------------------*/
/* 1. Display all existing workitems        			     */
/* 2. Display a selection of process instances using filter and      */
/*    threshold criteria.                                            */
/*-------------------------------------------------------------------*/

  //clear vector
  vwList.erase(vwList.begin(),vwList.end());

  cout << "\nDisplaying all workitems sorted by 'NAME ASC':\n"
       << endl;

  string WIListSort   = "NAME ASC";

  rc = service.QueryWorkitems( FmcjNoFilter,
  			       &WIListSort,
  			       FmcjNoThreshold,
  			       vwList );

  if (rc != FMC_OK)
  {
    cout << "Querying workitems failed with RC " << rc << " - "
         << result->MessageText() << endl;
    service.Logoff();
    // Deinit API environment
    FmcjGlobal::Disconnect();
    delete(result);
    return -1;
  }

  if ( vwList.size()>0 )
  {
    for( i=0; i<vwList.size(); i++ )
    {
      workitem = vwList[i];
      cout << "     " << workitem.Name() << endl;
    }
  }
  else cout << "     <none>" << endl;

  threshold = 10;

  cout << "\nDisplaying workitems with filter "
       << "'STATE=READY',\n" << "threshold " << threshold
       << " and sort criterium 'PROCESS_NAME ASC'.\n"
       << endl;

  string WIListFilter = "STATE IN (READY)";
  WIListSort   = "PROCESS_NAME ASC";

  vwList.erase(vwList.begin(),vwList.end());

  rc = service.QueryWorkitems( &WIListFilter,
  			       &WIListSort,
  			       &threshold,
  			       vwList );

  if (rc != FMC_OK)
  {
    cout << "Querying workitems failed with RC " << rc << " - "
         << result->MessageText() << endl;
    service.Logoff();
    // Deinit API environment
    FmcjGlobal::Disconnect();
    delete(result);
    return -1;
  }

  if ( vwList.size()>0 )
  {
    for( i=0; i<vwList.size(); i++ )
    {
      workitem = vwList[i];
      cout << "     " << workitem.Name() << endl;
    }
  }
  else cout << "     <none>" << endl;


/*-------------------------------------------------------------------*/
/* Logoff and deinit the API environment.               	     */
/*-------------------------------------------------------------------*/

  cout << "\nLogging off." << endl;
  rc = service.Logoff();
  cout << "Deinitializing API environment." << endl;
  rc = FmcjGlobal::Disconnect();
  delete(result);
  return 0;
}
